/*
Copyright (C) 2006 Evan Teran
                   eteran@alum.rit.edu

This program is free software; you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation; either version 2 of the License, or
(at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program; if not, write to the Free Software
Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.
*/

#ifndef I386ARCHPROCESSOR_20070312_H_
#define I386ARCHPROCESSOR_20070312_H_

#include <QVector>
#include <QStringList>
#include <QMap>
#include <QString>
#include "FunctionInfo.h"
#include "types.h"
#include "State.h"

class QTreeWidgetItem;
class QCatagoryList;
class Instruction;
class Operand;

class i386ArchProcessor {
public:
	i386ArchProcessor();
	
public:
	void setupRegisterView(QCatagoryList *catagoryList);
	void updateRegisterView(const QString &defaultRegionName, const State &state);
	QTreeWidgetItem *getRegisterItem(unsigned int index);
	QStringList updateInstructionInfo(BaseTypes::address_t address, const State &state);
	
private:
	void doUpdateRegister(QTreeWidgetItem *item, const QString &name, BaseTypes::reg_t value);
	QStringList doFunctionParamterResolution(const QString &symname, const State &state, int offset);

	void analyzeJump(const Operand *const *operands, const State &state, const Instruction &insn, QStringList &ret);
	void analyzeReturn(const Operand *const *operands, const State &state, const Instruction &insn, QStringList &ret);
	void analyzeCall(const Operand *const *operands, const State &state, const Instruction &insn, QStringList &ret);
	void analyzeOperands(const Operand *const *operands, const State &state, const Instruction &insn, QStringList &ret);
	
private:
	QVector<QTreeWidgetItem *>	m_RegisterViewItems;
	
	// TODO: move this somewhere more general
	QMap<QString, FunctionInfo> m_FunctionInfo;
	
	State m_LastState;
};

#endif

